package com.fr.plugin.decision.api.controller.platform;

import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.controller.DepartmentController;
import com.fr.decision.authority.data.Department;
import com.fr.decision.authority.data.User;
import com.fr.decision.authority.data.personnel.DepRole;
import com.fr.decision.webservice.Response;
import com.fr.decision.webservice.annotation.FinePathVariable;
import com.fr.decision.webservice.bean.authority.PrivilegeBean;
import com.fr.decision.webservice.bean.pack.BiPackageRowFilterBean;
import com.fr.decision.webservice.impl.privilege.RoleTypePrivilegeProcessFactory;
import com.fr.decision.webservice.v10.analysis.table.BiAnalysisTableService;
import com.fr.decision.webservice.v10.auth.BiPackageAuthorityService;
import com.fr.decision.webservice.v10.pack.BiPackageService;
import com.fr.decision.webservice.v10.user.PositionService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.api.bean.BIAuthorityBean;
import com.fr.plugin.decision.api.bean.RowFilterBean;
import com.fr.plugin.decision.api.service.APIService;
import com.fr.plugin.decision.api.utils.LogUtils;
import com.fr.report.constant.RoleType;
import com.fr.stable.StringUtils;
import com.fr.stable.query.QueryFactory;
import com.fr.third.jodd.util.StringUtil;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.RequestBody;
import com.fr.third.springframework.web.bind.annotation.RequestMapping;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.third.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author JianYe.Wang
 * @Data 2021/9/11 10:40
 * @Description TODO
 * @Version 10.0
 **/
@Controller
@RequestMapping("/api/v5/plugin/direct/auth")
public class AuthorityController {

    @RequestMapping(
            path = "/package/depost/{depostId}",
            method = RequestMethod.POST)
    @ResponseBody
    public Response updateDepPostAuth(HttpServletRequest req,
                                      @RequestBody List<BIAuthorityBean> biAuthorityBeans,
                                      @FinePathVariable("depostId") String depostId) throws Exception {
        String userId = UserService.getInstance().getCurrentUserId(req);
        List result = new ArrayList(biAuthorityBeans.size());
        for (BIAuthorityBean biAuthorityBean : biAuthorityBeans) {
            try {
                PrivilegeBean privilegeBean = APIService.getInstance().changeBIAuthBean2Privilege(biAuthorityBean);
                BiPackageAuthorityService.getInstance().updatePackageEntitiesByCarrier(userId,
                        "depost",
                        depostId,
                        privilegeBean);
                result.add(Response.success());
            } catch (Exception e) {
                result.add(Response.error("update dep authority", e.getMessage()));
            }
        }
        return Response.ok(result);
    }

    @RequestMapping(
            path = "/package/role/{roleId}",
            method = RequestMethod.POST)
    @ResponseBody
    public Response updateRoleAuth(HttpServletRequest req,
                                   @RequestBody List<BIAuthorityBean> biAuthorityBeans,
                                   @FinePathVariable("roleId") String roleId) throws Exception {
        String userId = UserService.getInstance().getCurrentUserId(req);
        List result = new ArrayList(biAuthorityBeans.size());
        for (BIAuthorityBean biAuthorityBean : biAuthorityBeans) {
            try {
                PrivilegeBean privilegeBean = APIService.getInstance().changeBIAuthBean2Privilege(biAuthorityBean);
                BiPackageAuthorityService.getInstance().updatePackageEntitiesByCarrier(userId,
                        "role",
                        APIService.getInstance().transferPost2Id(roleId),
                        privilegeBean);
                result.add(Response.success());
            } catch (Exception e) {
                result.add(Response.error("update role authority", e.getMessage()));
            }
        }
        return Response.ok(result);
    }

    @RequestMapping(
            path = "/package/user/{userId}",
            method = RequestMethod.POST)
    @ResponseBody
    public Response updateUserAuth(HttpServletRequest req,
                                   @RequestBody List<BIAuthorityBean> biAuthorityBeans,
                                   @FinePathVariable("userId") String userId) throws Exception {
        String currentUserId = UserService.getInstance().getCurrentUserId(req);
        List result = new ArrayList(biAuthorityBeans.size());
        for (BIAuthorityBean biAuthorityBean : biAuthorityBeans) {
            try {
                PrivilegeBean privilegeBean = APIService.getInstance().changeBIAuthBean2Privilege(biAuthorityBean);
                BiPackageAuthorityService.getInstance().updatePackageEntitiesByCarrier(currentUserId,
                        "user",
                        APIService.getInstance().transferUser2Id(userId),
                        privilegeBean);
                result.add(Response.success());
            } catch (Exception e) {
                result.add(Response.error("update user authority", e.getMessage()));
            }
        }
        return Response.ok(result);
    }

    @RequestMapping(
            path = "/row/depost/{depostId}",
            method = RequestMethod.POST)
    @ResponseBody
    public Response updateDepPostTableRowAuth(HttpServletRequest req,
                                              @RequestBody List<RowFilterBean> biAuthorityBeans,
                                              @FinePathVariable("depostId") String depostId) throws Exception {
        List result = new ArrayList(biAuthorityBeans.size());
        for (RowFilterBean rowFilterBean : biAuthorityBeans) {
            try {
                BiPackageRowFilterBean packageRowFilterBean = APIService.getInstance().constructRowFilter(rowFilterBean, RoleType.DEP.toInteger(), depostId);
                BiPackageRowFilterBean resultFilter;
                if (StringUtil.isBlank(packageRowFilterBean.getId())) {
                    resultFilter = BiPackageService.getInstance().addRowFilter(packageRowFilterBean);
                } else {
                    resultFilter = BiPackageService.getInstance().updateRowFilter(packageRowFilterBean);
                }
                result.add(resultFilter);
            } catch (Exception e) {
                FineLoggerFactory.getLogger().error(e.getMessage(), e);
                result.add(Response.error("update department row authority", e.getMessage()));
            }
        }
        return Response.ok(result);
    }

    @RequestMapping(
            path = "/row/role/{roleId}",
            method = RequestMethod.POST)
    @ResponseBody
    public Response updateRoleTableRowAuth(HttpServletRequest req,
                                           @RequestBody List<RowFilterBean> biAuthorityBeans,
                                           @FinePathVariable("roleId") String roleId) throws Exception {
        List result = new ArrayList(biAuthorityBeans.size());
        for (RowFilterBean rowFilterBean : biAuthorityBeans) {
            try {
                BiPackageRowFilterBean packageRowFilterBean = APIService.getInstance().constructRowFilter(rowFilterBean, RoleType.CUSTOM.toInteger(), APIService.getInstance().transferPost2Id(roleId));
                BiPackageRowFilterBean resultFilter;
                if (StringUtil.isBlank(packageRowFilterBean.getId())) {
                    resultFilter = BiPackageService.getInstance().addRowFilter(packageRowFilterBean);
                } else {
                    resultFilter = BiPackageService.getInstance().updateRowFilter(packageRowFilterBean);
                }
                result.add(resultFilter);
            } catch (Exception e) {
                FineLoggerFactory.getLogger().error(e.getMessage(), e);
                result.add(Response.error("update role row authority", e.getMessage()));
            }
        }
        return Response.ok(result);
    }

    @RequestMapping(
            path = "/row/user/{userId}",
            method = RequestMethod.POST)
    @ResponseBody
    public Response updateUserTableRowAuth(HttpServletRequest req,
                                           @RequestBody List<RowFilterBean> biAuthorityBeans,
                                           @FinePathVariable("userId") String userId) throws Exception {
        List result = new ArrayList(biAuthorityBeans.size());
        for (RowFilterBean rowFilterBean : biAuthorityBeans) {
            try {
                BiPackageRowFilterBean packageRowFilterBean = APIService.getInstance().constructRowFilter(rowFilterBean, RoleType.USER.toInteger(), APIService.getInstance().transferUser2Id(userId));
                BiPackageRowFilterBean resultFilter;
                if (StringUtil.isBlank(packageRowFilterBean.getId())) {
                    resultFilter = BiPackageService.getInstance().addRowFilter(packageRowFilterBean);
                } else {
                    resultFilter = BiPackageService.getInstance().updateRowFilter(packageRowFilterBean);
                }
                result.add(resultFilter);
            } catch (Exception e) {
                FineLoggerFactory.getLogger().error(e.getMessage(), e);
                result.add(Response.error("update role row authority", e.getMessage()));
            }
        }
        return Response.ok(result);
    }

    @RequestMapping(
            path = "/table/share/{tableName}/user",
            method = RequestMethod.GET)
    @ResponseBody
    public Response queryTableShareUser(HttpServletRequest req,
                                        @FinePathVariable("tableName") String tableName) throws Exception {
        List<User> userList = AuthorityContext.getInstance().getUserController().findByAuthority(
                tableName,
                RoleTypePrivilegeProcessFactory.getPrivilegeType("analysisTable").entityClass(),
                QueryFactory.create()
        );

        return Response.ok(userList.stream().map(user -> new HashMap() {{
            put("id", user.getId());
            put("name", user.getUserName());
        }}).collect(Collectors.toList()));
    }

    @RequestMapping(
            path = "/table/share/{tableName}/department",
            method = RequestMethod.GET)
    @ResponseBody
    public Response queryTableShareDepartment(HttpServletRequest req,
                                              @FinePathVariable("tableName") String tableName) throws Exception {
        DepartmentController controller = AuthorityContext.getInstance().getDepartmentController();
        List<DepRole> depRoleList = controller.findByAuthority(
                tableName,
                RoleTypePrivilegeProcessFactory.getPrivilegeType("analysisTable").entityClass(),
                QueryFactory.create()
        );

        return Response.ok(depRoleList.stream().map(depRole -> {
            HashMap<String, String> result = new HashMap<>();
            try {
                Department department = controller.getById(depRole.getDepartmentId());
                result.put("department", department.getName());
                result.put("departmentId", department.getId());

                String post = StringUtils.EMPTY, postId = depRole.getPostId();
                if (StringUtils.isNotEmpty(postId)) {
                    post = PositionService.getInstance().getPostNameById(postId);
                }
                result.put("post", post);
                result.put("postId", postId);
            } catch (Exception e) {
                LogUtils.error(e.getMessage(), e);
            }
            return result;
        }).collect(Collectors.toList()));
    }

    @RequestMapping(
            path = "/table/share/{tableName}",
            method = RequestMethod.POST)
    @ResponseBody
    public Response updateTableShareAuth(HttpServletRequest req,
                                         @RequestBody List<BIAuthorityBean> biAuthorityBeans,
                                         @FinePathVariable("tableName") String tableName) throws Exception {
        String userId = UserService.getInstance().getCurrentUserId(req);
        BiAnalysisTableService.getInstance().shareAnalysisTable(userId, tableName,
                biAuthorityBeans
                        .stream()
                        .map(APIService.getInstance()::changeBIAuthBean2SharePrivilege)
                        .collect(Collectors.toList())
        );
        return Response.success();
    }
}
