package com.fr.plugin.decision.api.excel;

import com.finebi.foundation.api.bean.BeanHelper;
import com.finebi.widget.api.cell.user.FineAuthorityUser;
import com.fr.invoke.Reflect;
import com.fr.io.utils.ResourceIOUtils;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.api.utils.LogUtils;
import com.fr.stable.StableUtils;
import com.fr.stable.project.ProjectConstants;

import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author JianYe.Wang
 * @Data 2021/10/25 16:06
 * @Description TODO
 * @Version 10.0
 **/
public class ExcelReportSelector {

    private static volatile ExcelReportSelector service;
    private static final String FILE_NAME = "excelReportIndex.json";
    public final static String EXCEL_FILE_PATH = "excelModules";


    private ExcelReportSelector() {
    }

    public static ExcelReportSelector getInstance() {
        if (null == service) {
            synchronized (ExcelReportSelector.class) {
                if (null == service) {
                    service = new ExcelReportSelector();
                }
            }
        }

        return service;
    }

    public boolean isExcelReport(String reportId) {
        return getReportIds().contains(reportId);
    }

    public Set<String> getReportIds() {
        try {
            String filePath = StableUtils.pathJoin(EXCEL_FILE_PATH, FILE_NAME);
            if (!ResourceIOUtils.exist(filePath)) {
                filePath = StableUtils.pathJoin("assets", filePath);
            }
            if (!ResourceIOUtils.exist(filePath)) {
                LogUtils.error("load excel json file error!");
                return new HashSet<>();
            }
            byte[] bytes = ResourceIOUtils.readBytes(filePath);
            ExcelReportIndexConf excelReportIndexConf = BeanHelper.excludeNullMapper().readValue(bytes, ExcelReportIndexConf.class);
            Set reportIds = excelReportIndexConf.getReportIds() == null ? new HashSet<>() : excelReportIndexConf.getReportIds();
            LogUtils.info("ExcelReports:{}", reportIds);
            return reportIds;
        } catch (Exception e) {
            LogUtils.error(e, "PublicLinkExportSettingDao getAll error ! ");
            return new HashSet<>();
        }
    }

    /**
     * 获取电子表格中使用到的表和字段名
     *
     * @param reportId
     * @param fineAuthorityUser
     * @return
     */
    public Map<String, List<String>> getReportTableFieldMap(String reportId, FineAuthorityUser fineAuthorityUser) {
        String path = StableUtils.pathJoin(StableUtils.pathJoin(ProjectConstants.ASSETS_NAME, EXCEL_FILE_PATH), reportId + ".xlsx");
        InputStream inputStream = ResourceIOUtils.read(path);
        if (inputStream == null) {
            throw new IllegalArgumentException("Excel file is absent ! ExcelFileName is " + reportId + "! ");
        }
        //FineAuthorityUser fineAuthorityUser = AuthorityUserCreator.create(userId);
        try {
            Reflect workbook = Reflect.on(StableUtils.classForName("com.aspose.cells.Workbook"))
                    .create(inputStream);
            Reflect paramAndFilter = Reflect.on(StableUtils.classForName("com.finebi.plugin.excel.report.ExcelParamAndFilter"))
                    .create();
            Reflect translator = Reflect.on(StableUtils.classForName("com.finebi.plugin.excel.aspose.AsposeExcelTranslator"))
                    .create(new Object[]{workbook.get(), fineAuthorityUser, paramAndFilter.get(), true})
                    .call("preTranslate");

            HashMap infoMap = translator.get("infoMap");
            List<String> fields = (List<String>) infoMap.values()
                    .stream()
                    .map(o -> Reflect.on(o).get("id"))
                    .collect(Collectors.toList());
            Set<String> tableNames = translator.get("tableNameSet");

            // infoMap 中返回的 tableName 字段不准确，只能使用 tableNameSet，字段因为是先查询数据集字段，再与改使用字段做匹配，所以可以就先全部汇总返回
            /*return (HashMap<String, List<String>>) infoMap.values().stream()
                    .collect(Collectors.groupingBy(o -> Reflect.on(o).get("tableName"),  Collectors.mapping(o -> Reflect.on(o).get("id"), Collectors.toList())));*/
            return tableNames.stream().collect(Collectors.toMap(o -> o, o -> fields));
        } catch (ClassNotFoundException e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
            throw new IllegalArgumentException("Excel plugin version not match!");
        }
    }
}
