package com.fr.plugin.decision.api.service.consanguinity;

import com.finebi.common.api.vo.pack.FineBusinessPackage;
import com.finebi.common.api.vo.table.*;
import com.finebi.common.exception.FineEngineException;
import com.finebi.dashboard.api.bean.report.ReportBean;
import com.finebi.dashboard.api.bean.report.consanguinity.*;
import com.finebi.dashboard.api.bean.widget.WidgetBean;
import com.finebi.dashboard.api.bean.widget.export.ExcelReportWidgetBean;
import com.finebi.dashboard.api.bean.widget.multitab.MultiTabWidgetBean;
import com.finebi.dashboard.api.cell.DashboardCellCreator;
import com.finebi.dashboard.api.cell.DashboardTableCell;
import com.finebi.dashboard.impl.widget.visitor.WidgetBeanTablesNameVisitor;
import com.finebi.utils.CollectionUtils;
import com.finebi.widget.api.cell.WidgetCellCreator;
import com.finebi.widget.api.cell.WidgetCellProvider;
import com.finebi.widget.api.cell.conf.TableWidgetCell;
import com.finebi.widget.api.cell.user.FineAuthorityUser;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.api.excel.ExcelReportSelector;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;
import java.util.function.BiFunction;
import java.util.stream.Collectors;

/**
 * 添加下对tab组件的支持
 */
public class XReportConsanguinityHelper {
    private Map<String, TableNameDependenceInfo> engineDependenceInfoMap = new HashMap<>();
    private FineAuthorityUser user;
    private Map<String, WidgetBean> widgets;

    public XReportConsanguinityHelper(FineAuthorityUser user) {
        this.user = user;
    }

    public ReportConsanguinityBean getReportConsanguinityByReportId(String reportId) {
        try {
            Optional<ReportBean> reportBeanByReportId = DashboardCellCreator.reportBean().getReportBeanByReportId(reportId);
            DashboardTableCell tableCell = DashboardCellCreator.table();
            if (reportBeanByReportId.isPresent()) {
                ReportBean reportBean = reportBeanByReportId.get();
                ReportConsanguinityBean reportConsanguinityBean = new ReportConsanguinityBean();
                reportConsanguinityBean.setReportId(reportId);
                reportConsanguinityBean.setReportName(reportBean.getReportName());
                widgets = reportBean.getWidgets();
                if (CollectionUtils.isNotEmptyMap(widgets)) {
                    List<WidgetConsanguinityBean> widgetConsanguinityBeans = widgets.values()
                            .stream()
                            .map((widget) -> this.getWidgetConsanguinity(tableCell, widget))
                            .collect(Collectors.toList());
                    reportConsanguinityBean.setWidgets(widgetConsanguinityBeans);
                }

                return reportConsanguinityBean;
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e, e.getMessage());
        }

        return new ReportConsanguinityBean();
    }

    @NotNull
    private WidgetConsanguinityBean getWidgetConsanguinity(DashboardTableCell tableCell, WidgetBean widget) {
        WidgetConsanguinityBean widgetConsanguinityBean = new WidgetConsanguinityBean();
        widgetConsanguinityBean.setWidgetId(widget.getWid());
        widgetConsanguinityBean.setWidgetName(widget.getName());

        // tab组件 WidgetBeanTablesNameVisitor 返回 null
        if (widget instanceof MultiTabWidgetBean) {
            ((MultiTabWidgetBean) widget).getTabs().forEach(widgetTabBean ->
                    widgetTabBean.getWidgets().keySet().forEach(key -> {
                                getWidgetConsanguinity(tableCell, widgets.get(key));
                            }
                    )
            );
            return widgetConsanguinityBean;
        }
        Collection<String> tableNames;
        WidgetUsedTableFields tableFields;
        if (widget instanceof ExcelReportWidgetBean) {
            Map<String, List<String>> tableFiledMap = ExcelReportSelector.getInstance().getReportTableFieldMap(((ExcelReportWidgetBean) widget).getExcelFileName(), user);
            tableNames = tableFiledMap.keySet();
            tableFields = new WidgetUsedTableFields(tableFiledMap, this.user);
        } else {
            tableNames = widget.accept(new WidgetBeanTablesNameVisitor(tableCell));
            tableFields = new WidgetUsedTableFields(widget, this.user);
        }
        List<TableConsanguinityBean> collect = tableNames
                .stream()
                .map((table) -> this.getTableConsanguinity(table, tableFields))
                .filter(Objects::nonNull)
                .collect(Collectors.toList());
        widgetConsanguinityBean.setTables(collect);
        return widgetConsanguinityBean;
    }

    @Nullable
    private TableConsanguinityBean getTableConsanguinity(String table, WidgetUsedTableFields widgetUsedTableFields) {
        try {
            WidgetCellProvider cellProviderByTable = WidgetCellCreator.getCellProviderByTable(table);
            return this.getTableConsanguinity(cellProviderByTable, table, widgetUsedTableFields::isUsedTableField);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e, e.getMessage());
            return null;
        }
    }

    @Nullable
    private TableConsanguinityBean getTableConsanguinity(WidgetCellProvider provider, String currentTable, BiFunction<String, String, Boolean> useFieldFunction) {
        try {
            //Optional<FineBusinessPackage> optional = provider.packageCell().getPackageByTableName(currentTable, this.user);
            Optional<FineBusinessPackage> optional = provider.packageCell().getPackageByTableNameWithoutAuth(currentTable);
            if (optional.isPresent()) {
                FineBusinessPackage fineBusinessPackage = optional.get();
                TableWidgetCell tableWidgetCell = provider.analysisTableCell();
                //Optional<FineBusinessTable> singleTable = tableWidgetCell.getSingleTable(currentTable, this.user);
                Optional<FineBusinessTable> singleTable = tableWidgetCell.getSingleTableWithoutAuth(currentTable);
                if (singleTable.isPresent()) {
                    return singleTable.get().accept(this.getFineTableVisitor(provider, useFieldFunction, fineBusinessPackage));
                }
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e, "get Origin tables error ! current table name is : " + currentTable + " !");
        }

        return null;
    }

    @NotNull
    private FineBusinessTableVisitor<TableConsanguinityBean> getFineTableVisitor(final WidgetCellProvider provider, final BiFunction<String, String, Boolean> useFieldFunction, final FineBusinessPackage fineBusinessPackage) {
        return new FineBusinessTableVisitor<TableConsanguinityBean>() {
            public TableConsanguinityBean visit(FineAnalysisTable table) throws Exception {
                AnalysisTableConsanguinityBean analysis = new AnalysisTableConsanguinityBean();
                analysis.setFatherTables(getAnalysisOriginTables(provider, table.getName()));
                analysis.setPackageId(fineBusinessPackage.getId());
                analysis.setPackageName(fineBusinessPackage.getName());
                analysis.setTableName(table.getName());
                analysis.setTransferName(table.getTransferName());
                analysis.setFields(this.createFields(table));
                return analysis;
            }

            public TableConsanguinityBean visit(FineDBBusinessTable table) throws Exception {
                DataBaseTableConsanguinityBean tableConsanguinityBean = new DataBaseTableConsanguinityBean();
                tableConsanguinityBean.setConnectionName(table.getConnectionName());
                tableConsanguinityBean.setDbTableName(table.getTableName());
                tableConsanguinityBean.setPackageId(fineBusinessPackage.getId());
                tableConsanguinityBean.setPackageName(fineBusinessPackage.getName());
                tableConsanguinityBean.setTableName(table.getName());
                tableConsanguinityBean.setTransferName(table.getTransferName());
                tableConsanguinityBean.setFields(this.createFields(table));
                return tableConsanguinityBean;
            }

            public TableConsanguinityBean visit(FineSQLBusinessTable table) throws Exception {
                SqlTableConsanguinityBean tableConsanguinityBean = new SqlTableConsanguinityBean();
                tableConsanguinityBean.setConnectionName(table.getConnName());
                tableConsanguinityBean.setSql(table.getSql());
                tableConsanguinityBean.setPackageId(fineBusinessPackage.getId());
                tableConsanguinityBean.setPackageName(fineBusinessPackage.getName());
                tableConsanguinityBean.setTableName(table.getName());
                tableConsanguinityBean.setTransferName(table.getTransferName());
                tableConsanguinityBean.setFields(this.createFields(table));
                return tableConsanguinityBean;
            }

            public TableConsanguinityBean visit(FineServerBusinessTable table) throws Exception {
                ServerTableConsanguinityBean tableConsanguinityBean = new ServerTableConsanguinityBean();
                tableConsanguinityBean.setServerTableName(table.getSourceName());
                tableConsanguinityBean.setPackageId(fineBusinessPackage.getId());
                tableConsanguinityBean.setPackageName(fineBusinessPackage.getName());
                tableConsanguinityBean.setTableName(table.getName());
                tableConsanguinityBean.setTransferName(table.getTransferName());
                tableConsanguinityBean.setFields(this.createFields(table));
                return tableConsanguinityBean;
            }

            public TableConsanguinityBean visit(FineExcelBusinessTable table) throws Exception {
                ExcelTableConsanguinityBean tableConsanguinityBean = new ExcelTableConsanguinityBean();
                tableConsanguinityBean.setExcelName(table.getBaseAttach().getFileName());
                tableConsanguinityBean.setPackageId(fineBusinessPackage.getId());
                tableConsanguinityBean.setPackageName(fineBusinessPackage.getName());
                tableConsanguinityBean.setTableName(table.getName());
                tableConsanguinityBean.setTransferName(table.getTransferName());
                tableConsanguinityBean.setFields(this.createFields(table));
                return tableConsanguinityBean;
            }

            public TableConsanguinityBean visit(FineOldETLTable fineOldETLTable) throws Exception {
                throw new UnsupportedOperationException("FineOldETLTable unsupport Report Consanguinity !");
            }

            private List<FieldConsanguinityBean> createFields(FineBusinessTable table) {
                return (CollectionUtils.isEmpty(table.getFields()) ? Collections.emptyList() : table.getFields()
                        .stream()
                        .filter((fineBusinessField) -> useFieldFunction.apply(table.getName(), fineBusinessField.getId()))
                        .map((fineBusinessField) -> {
                            FieldConsanguinityBean fieldConsanguinityBean = new FieldConsanguinityBean();
                            fieldConsanguinityBean.setFieldId(fineBusinessField.getId());
                            fieldConsanguinityBean.setFieldName(fineBusinessField.getName());
                            fieldConsanguinityBean.setTransferName(fineBusinessField.getTransferName());
                            return fieldConsanguinityBean;
                        })
                        .collect(Collectors.toList()));
            }
        };
    }

    private Set<TableConsanguinityBean> getAnalysisOriginTables(WidgetCellProvider provider, String currentTable) throws FineEngineException {
        TableNameDependenceInfo tableNameDependenceInfo;
        if (this.engineDependenceInfoMap.containsKey(provider.getEngineVO().getName())) {
            tableNameDependenceInfo = this.engineDependenceInfoMap.get(provider.getEngineVO().getName());
        } else {
            tableNameDependenceInfo = provider.analysisTableCell().getTableDependenceSupplier().get();
            this.engineDependenceInfoMap.put(provider.getEngineVO().getName(), tableNameDependenceInfo);
        }

        Set<String> fatherTables = tableNameDependenceInfo.getTableFatherNamesMap().get(currentTable);
        return (CollectionUtils.isEmpty(fatherTables) ? Collections.emptySet() : fatherTables
                .stream()
                .map((fatherTable) -> this.getTableConsanguinity(provider, fatherTable, (tableName, fieldId) -> true))
                .collect(Collectors.toSet()));
    }
}
